@@ -119,7 +119,7 @@ class LinuxDevice(object):
     '''
     A base class for common linux devices.
     '''
-    version = '1.0.9'
+    version = '1.1.5'
 
     LOG_EMERG = 0   # not used
     LOG_ALERT = 1   # not used
@@ -218,7 +218,9 @@ class LinuxDevice(object):
             self._preconfig()
 
         if self["type"] == 'Linux':
-            self.log("%s = %s\n" %(self.name, json.dumps(self.attributes, indent=4)))
+            tmp = copy.deepcopy(self.attributes)
+            tmp["password"] = "*******"
+            self.log("%s = %s\n" %(self.name, json.dumps(tmp, indent=4)))
         #self.x_get_interface()
         #self.log("interface = %s\n" %(json.dumps(self.interface, indent=4)))
 
@@ -714,10 +716,7 @@ class LinuxDevice(object):
         """
         Get members of the object.
         """
-        if name not in self.attributes:
-            return None
-        else:
-            return self.attributes[name]
+        return self.attributes.get(name, None)
 
     def __setitem__(self, name, value):
         """
