@@ -16,6 +16,7 @@ except ImportError:
 import sys, os, time
 import collections
 import tempfile
+import copy
 
 class OvsHost(linux.LinuxDevice):
     '''
@@ -30,6 +31,7 @@ class OvsHost(linux.LinuxDevice):
         '''
 
         linux.LinuxDevice.__init__(self, server, **kwargs)
+        self.device = []
 
         # start the docker if it's not running
         num = self.cmd('ps -ef | grep -w ovs-vswitchd | grep -v grep | wc -l')
@@ -76,7 +78,7 @@ class OvsHost(linux.LinuxDevice):
             self.get_file('/usr/local/var/log/openvswitch/ovs-vswitchd.log',
                     "%s_ovs-vswitched.log" %(self["name"]))
 
-    def add_br (self, bridge, *args, **kwargs):
+    def add_br (self, name, *args, **kwargs):
         '''
         Add a bridge and build the subnet.
 
@@ -98,38 +100,40 @@ class OvsHost(linux.LinuxDevice):
                     "ip": "10.208.1.11/24"}
 
         '''
-        bridge_name = bridge.pop("name", None) #get bridge name
-        ip = bridge.pop("ip", []) #get the ip configuration of the bridge
-        ports = bridge.pop("port", []) # get the ports list
-        # Create the bridge
-        command = 'ovs-vsctl --may-exist add-br %s' %(bridge_name)
-        if bridge:
+        ip = kwargs.pop("ip", []) #get the ip configuration of the bridge
+        ports = kwargs.pop("port", []) # get the ports list
+        # Create the kwargs
+        command = 'ovs-vsctl --may-exist add-br %s' %(name)
+        if kwargs:
             # If there is parameters, for example datapath_type, etc.
-            command += ' -- set bridge %s' %(bridge_name)
-            for k,v in bridge.items():
+            command += ' -- set bridge %s' %(name)
+            for k,v in kwargs.items():
                 command += ' %s=%s' %(k,v)
+                self[k] = v
         self.cmd(command) #execut the command.
+
+        # delete the current offlow when it's created
+        #if self["fail-mode"] == "secure":
+        #    self.cmd("ovs-ofctl del-flows %s" %(name))
+
         if ip:
             # Configure the ip address for the address for route
-            self.cmd('ip address add %s dev %s' %(ip, bridge_name))
-            self.cmd('ip link set %s up' %(bridge_name))
-            self.cmd('ovs-appctl ovs/route/add %s %s' %(ip, bridge_name))
+            self.cmd('ip address add %s dev %s' %(ip, name))
+            self.cmd('ip link set %s up' %(name))
+            self.cmd('ovs-appctl ovs/route/add %s %s' %(ip, name))
 
-        # Add self-port if any
+        # Add self-port if any, it's port = xxx, or *args,
         ports = ports if (isinstance(ports, list)) else [ports]
+        ports = ports if (not args) else ports.extend(args)
         for p in ports:
-            self.add_port(bridge_name, **p)
-
-        # Add remote-device and it's peer self-port
-        #for d in args:
-        #    print("d: " %(d))
-        #    self.add_device(bridge_name, **d)
+            self.add_port(name, **p)
 
     def add_port (self, bridge_name, name, **kwargs):
         '''
         Add self-port and do some configuration if necessary.
         '''
         command = 'ovs-vsctl add-port %s %s' %(bridge_name, name)
+        ip = kwargs.pop("ip", []) #get the ip configuration of the bridge
         if kwargs:
             # If there is parameters, for example type=vxlan, etc.
             command += ' -- set interface %s' %(name)
@@ -137,7 +141,79 @@ class OvsHost(linux.LinuxDevice):
                 command += ' %s=%s' %(k,v)
         self.cmd(command) #execut the command.
 
-    def add_vm (self, mask = None,
+        #If it's a normal interface, flush it to release its original ip address.
+        if (kwargs.get("type", "normal") == "normal"):
+            self.cmd("ip addr flush dev %s" %(name))
+
+        if ip:
+            # Configure the ip address for the address for route
+            self.cmd('ip link set %s up' %(name))
+            self.cmd('ip address add %s dev %s' %(ip, name))
+            #self.cmd('ovs-appctl ovs/route/add %s %s' %(ip, name))
+            if self["fail-mode"] == "secure":
+                # Confiure the default offlow for self port with IP.
+                self.ofctl_add_flow(bridge_name,
+                    "priority=65535,arp,arp_tpa=%s actions=normal" %(ip.split("/")[0]),
+                    "priority=1,ip,nw_dst=%s actions=normal" %(ip.split("/")[0]))
+
+    def add_device (self, bridge_name, *args, **kwargs):
+        '''
+        connect devices to ovs, now only support container.
+        '''
+        # Add remote-device and it's peer self-port
+        for arg in args:
+            # allocate and add the device into the list
+            self.device.append(arg["name"])
+
+            dtype = arg.get("type", None)
+            if dtype == "container":
+                self.__add_container(bridge_name, **arg)
+
+                macaddr = self.cmd(
+                    "docker exec -it %s ip link show %s | grep link/ether"
+                    %(arg["name"], arg["interface"]), log_level = 4)
+                # be compatible with the old version.
+                arg["mac"] = macaddr.split()[1]
+                arg["ofport"] = self.ovs_get_container_ofport(arg["name"])
+            elif dtype == "qemu-system-x86_64":
+                self.__add_vm(**arg)
+            else:
+                self.log("device type %s is not supported now..." %(dtype))
+
+            # Configure offlow for the same subnet
+            if self["fail-mode"] == "secure":
+                ipaddr = ipaddress.ip_interface(arg["ip"])
+                self.ofctl_add_flow(bridge_name,
+                    "priority=65535,arp,arp_tpa=%s actions=output:%s"
+                    %(ipaddr.ip, arg["ofport"]),
+                    "priority=60000,ip,nw_src=%s,nw_dst=%s actions=output:%s" 
+                    %(ipaddr.network, ipaddr.ip, arg["ofport"]))
+
+    def __add_container (self, bridge_name,
+            name = "con1",
+            type = "docker",
+            host_dir = "/var/shared",
+            container_dir = "/var/shared", 
+            interface = "eth1",
+            ip = "",
+            vlan = None,
+            **kwargs):
+        '''
+        Create a container and connect it to the bridge: bridge_name.
+        '''
+        #创建容器, 设置net=none可以防止docker0默认网桥影响连通性测试
+        self.cmd('docker run --privileged -d --name %s --net=none -v %s:%s -it centos'
+                %(name, host_dir, container_dir))
+        #self.cmd(' docker exec -it con11 rpm -ivh /var/shared/tcpdump-4.9.3-1.el8.x86_64.rpm ')
+        self.cmd('ovs-docker add-port %s %s %s --ipaddress=%s'
+                %(bridge_name, interface, name, ip))
+
+        # Configure vlan on self-port if the device has vlan configured.
+        if vlan:
+            self.cmd('ovs-vsctl set port %s tag=%d'
+                    %(ovs_get_container_port(bridge_name, name = name), vlan))
+
+    def __add_vm (self, mask = None,
             type = "qemu-system-x86_64",
             name = "vm1",
             cpu = "host", 
@@ -179,16 +255,16 @@ class OvsHost(linux.LinuxDevice):
             for c in chardev:
                 # Parse the chardev to find its type.
                 tmp_dic = {}
-                for a in c.split(","):
-                    k,v = ("=" in a) and a.split("=") or (a, True)
+                for a in c.split(",", 1):
+                    k,v = ("=" in a) and a.split("=", 1) or (a, True)
                     tmp_dic[k.strip()] = (isinstance(v, str)) and v.strip() or v
                 all_chardev[tmp_dic["id"]] = "-chardev %s" %(c)
             #print(all_chardev)
             for n in netdev:
                 # Parse the netdev to find its type.
                 tmp_dic = {}
-                for a in n.split(","):
-                    k,v = ("=" in a) and a.split("=") or (a, True)
+                for a in n.split(",", 1):
+                    k,v = ("=" in a) and a.split("=", 1) or (a, True)
                     tmp_dic[k.strip()] = (isinstance(v, str)) and v.strip() or v
                 if ("chardev" in tmp_dic):
                     all_netdev[tmp_dic["id"]] = " %s -netdev %s"\
@@ -199,8 +275,8 @@ class OvsHost(linux.LinuxDevice):
             for d in device:
                 # Parse the device to find its type.
                 tmp_dic = {}
-                for a in d.split(","):
-                    k,v = ("=" in a) and a.split("=") or (a, True)
+                for a in d.split(",", 1):
+                    k,v = ("=" in a) and a.split("=", 1) or (a, True)
                     tmp_dic[k.strip()] = (isinstance(v, str)) and v.strip() or v
                 cmd += " %s -device %s" %(all_netdev[tmp_dic["netdev"]], d)
         print(cmd)
@@ -208,34 +284,44 @@ class OvsHost(linux.LinuxDevice):
         #self.cmd("whoami")
         #self.cmd(cmd)
 
-    def add_devices (self, bridge_name, *args, **kwargs):
+    def add_gateway (self, bridge_name, gw, *args):
         '''
-        connect devices to ovs, now only support container.
+        Add some flows to ofproto
         '''
-        # Add remote-device and it's peer self-port
+        vxlan_port = None # Get the vxlan at first.
         for d in args:
-            dtype = d.get("type", None)
-            if dtype == "docker":
-                # Create a container and the self-port
-                ports = d.pop("port", []) # get the ports list
-                for p in ports:
-                    self.cmd('docker run --privileged -d --name %s -v %s:%s -it centos'
-                            %(p["name"], p.get("host_dir", "/var/shared"),
-                              p.get("container_dir", "/var/shared")))
+            if d["type"] == "vxlan":
+                vxlan_port = self.ovs_get_interface(d["name"], "ofport")
+                break
+        for d in args:
+            if d["type"] != "vxlan":
+                if ((d["name"] not in self.device) and (not vxlan_port)):
+                    # d is not on host, but there is no vxlan given.
+                    self.log("Error: there is no vxlan to remote device: %s!"
+                            %(d["name"]), bg_color = "red")
+                    return
+                self.ofctl_add_flow(bridge_name,
+                    "priority=10,ip,nw_dst=%s,action=mod_dl_src:%s,"
+                    "mod_dl_dst:%s,dec_ttl,output:%s"
+                    %(d["ip"].split("/")[0], gw["mac"], d["mac"],
+                        (d["name"] in self.device) and d["ofport"] or vxlan_port))
 
-                    #Allocate the interface/ipaddress and connect it to ovs bridge
-                    self.cmd('ovs-docker add-port %s %s %s --ipaddress=%s'
-                            %(bridge_name, p["interface"], p["name"], p["ip"]))
-            elif dtype == "qemu-system-x86_64":
-                # Create ifup/ifdown script
-                self.add_vm(**d)
-            else:
-                self.log("device type %s is not supported now..." %(dtype))
-
-            # Configure vlan on self-port if the device has vlan configured.
-            #if d.get("vlan", None):
-            #    self.cmd('ovs-vsctl set port %s tag=%d'
-            #            %(__ovs_get_port(bridge_name, name = name), d["vlan"]))
+    def add_vtep (self, bridge_name, vtep, *args):
+        '''
+        Add some flows remote devices(*args): 从本主机上到达所有*args的包，全
+        部经由vtep
+        '''
+        if self["fail-mode"] != "secure":
+            #Don't need configure vtep explicitly in standalone mode.
+            return
+        for arg in args:
+            ip = ipaddress.ip_interface(arg["ip"])
+            vxlan_port = self.ovs_get_interface(vtep["name"], "ofport")
+            self.ofctl_add_flow(bridge_name,
+                    "priority=65535,arp,arp_tpa=%s actions=output:%s"
+                    %(ip.ip, vxlan_port),
+                    "priority=60000,ip,nw_src=%s,nw_dst=%s actions=output:%s" 
+                    %(ip.network, ip.ip, vxlan_port))
 
     def del_br (self, *args, **kwargs):
         '''
@@ -249,100 +335,105 @@ class OvsHost(linux.LinuxDevice):
             ports = self.cmd("ovs-vsctl list-ports %s" %(b.strip()))
             for p in StringIO(ports).readlines():
                 # delete all the devices(container/vm/physical) connectting to.
-                external_ids = self.__get_external_ids(p.strip())
-                #print("external_ids: %s" %(external_ids))
-                if (external_ids.get("container_id", None)):
-                    self.cmd('docker stop %s' %(external_ids["container_id"]))
-                    self.cmd('docker rm %s' %(external_ids["container_id"]))
+                external_ids = self.ovs_get_interface(p.strip(), "external_ids")
+                external_ids = external_ids.strip("{} \t\r\n")
+                if ("=" in external_ids):
+                    # Parse external_ids: {container_id=con13, container_iface=eth1}
+                    i = {}
+                    for a in external_ids.split(",", 1):
+                        k,v = a.split("=", 1)
+                        i[k.strip()] = v.strip()
+                    if (i.get("container_id", None)):
+                        self.cmd('docker stop %s' %(i["container_id"]))
+                        self.cmd('docker rm %s' %(i["container_id"]))
             self.cmd("ovs-vsctl del-br %s" %(b.strip()))
         self.cmd('ovs-vsctl show')
         self.cmd('ovs-ctl stop')
 
-    def __get_external_ids (self, port = None):
-        '''
-        parse external_ids and return the dictionary.
-        '''
-        external_ids = {}
-        command = "ovs-vsctl list interface"
-        if port:
-            command += " %s" %(port)
-        command += " | awk -F: '/container_id/ {print $2}'"
-
-        result = self.cmd(command)
-        for line in StringIO(result).readlines():
-            for a in line.strip("{} \t\r\n").split(","):
-                k,v = a.split("=")
-                external_ids[k.strip()] = v.strip()
-        #print("external_ids: %s" %(external_ids))
-        return external_ids
-
-    def __ovs_get_port (self, bridge, name, **kwargs):
+    def ovs_get_interface (self, interface, attribute = None):
         '''
         Get a self-port which connect to the kwargs["name"]
         '''
-        ports = self.cmd("ovs-vsctl list-ports %s" %(bridge))
-        for p in StringIO(ports).readlines():
-            external_ids = self.__get_external_ids(p.strip())
-            if (external_ids.get("container_id", None) == name):
-                return p.strip()
+
+        if attribute:
+            result = self.cmd("ovs-vsctl get interface %s %s"
+                    %(interface, attribute))
+            return result.strip()
+
+        i = {}
+        result = self.cmd("ovs-vsctl list interface %s" %(interface),
+                log_level=4)
+        for p in StringIO(result).readlines():
+            k,v = p.split(":", 1)
+            i[k.strip()] = v.strip()
+
+        #return attribute and i.get(attribute, None) or i
+        return i
+
+    def ovs_get_container_ofport (self, name, **kwargs):
+        '''
+        Get a self-port which connect to the kwargs["name"]
+        '''
+
+        bridges = self.cmd("ovs-vsctl list-br", log_level=4)
+        for b in StringIO(bridges).readlines():
+            ports = self.cmd("ovs-vsctl list-ports %s" %(b.strip()), log_level=4)
+            for p in StringIO(ports).readlines():
+                i = self.ovs_get_interface(p.strip())
+                if (name in i.get("external_ids", None)):
+                    return i.get("ofport", None)
         return None
 
-    def ovs_add_flows (self, bridge, *args, **kwargs):
+    def ovs_get_container_port (self, name, **kwargs):
+        '''
+        Get a self-port which connect to the kwargs["name"]
+        '''
+
+        bridges = self.cmd("ovs-vsctl list-br", log_level=4)
+        for b in StringIO(bridges).readlines():
+            ports = self.cmd("ovs-vsctl list-ports %s" %(b.strip()), log_level=4)
+            for p in StringIO(ports).readlines():
+                i = self.ovs_get_interface(p.strip())
+                if (name in i.get("external_ids", None)):
+                    return i.get("name", None)
+        return None
+
+    def ofctl_add_flow (self, bridge_name, *args, **kwargs):
         '''
         Add some flows to ofproto
         '''
-
         for table in args:
             table = filter(lambda x: (x.strip()) and (x.strip()[0] != '#'),
                     StringIO(table.strip()).readlines())
             for l in table:
                 # remove the line starting with '#'
-                self.cmd('ovs-ofctl add-flow %s "%s"'
-                        %(bridge["name"], l.strip()))
+                l = l.strip()
+                if l[0] !=  "#":
+                    self.cmd('ovs-ofctl add-flow %s "%s"' %(bridge_name, l))
         return None
 
-    def ovs_get_topology (self, *args, **kwargs):
+    def ping_test (self, src, dst):
         '''
-        Destroy the topology:
-
-            * Destroy the OVS and all its bridges.
-            * Destroy all the devices connected to it if given in the *args.
+        Add some flows to ofproto
         '''
-        # delete all the bridges in the OVS
-        #topology = {"name": self["name"], "bridges": [] }
-        topology = []
-        bridges = self.cmd("ovs-vsctl list-br")
-        for b in StringIO(bridges).readlines():
-            #print("Bridge %s" %(b.strip()))
-            topology.append("Bridge %s" %(b.strip()))
-            #bridge = {"name": b.strip(), "ports": []}
-            ports = self.cmd("ovs-vsctl list-ports %s" %(b.strip()))
-            for p in StringIO(ports).readlines():
-                external_ids = self.__get_external_ids(p.strip())
-                if (not external_ids):
-                    topology.append("    Port %s, external_ids: %s"
-                            %(p.strip(), external_ids.strip()))
+        if src["type"] == "container":
+            result = self.cmd('docker exec -it %s ping %s -c 1' %(src["name"],
+                dst["ip"].split("/")[0]))
+        else:
+            result = None
 
-                options = self.cmd(
-                 "ovs-vsctl list interface %s | awk -F: '/options/ {print $2}'"
-                  %(p.strip()))
-                if (options.strip() != "{}"):
-                    topology.append("    Port %s, options: %s"
-                            %(p.strip(), options.strip()))
-
-                link_speed = self.cmd(
-                 "ovs-vsctl list interface %s | awk -F: '/link_speed/ {print $2}'"
-                  %(p.strip()))
-                if (options.strip() != "[]"):
-                    topology.append("    Port %s, link_speed: %s"
-                            %(p.strip(), link_speed.strip()))
-                #port = {"name": p.strip(), "interface": p.strip()}
-                #print("    Port %s %s" %(p.strip(), external_ids.strip()))
-                #bridge["ports"].append(port)
-            #topology["bridges"].append(bridge)
-        #self.log("container: %s\n" %(json.dumps(topology, indent=4)))
-        for i in topology:
-            print(i)
+        if "received, 0% packet loss," in result:
+            self.log("PASS: %s ping %s, %s -> %s!" %(src["name"],
+                dst["name"], src["ip"].split("/")[0],
+                dst["ip"].split("/")[0]),
+                bg_color = "green")
+            return True
+        else:
+            self.log("FAIL: %s ping %s, %s -> %s!" %(src["name"],
+                dst["name"], src["ip"].split("/")[0],
+                dst["ip"].split("/")[0]),
+                bg_color = "red")
+            return False
 
 if __name__ == '__main__':
     '''
